#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
    PsychoPy routine for settings if BIDS class
"""

# Part of the PsychoPy library
# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2021 Open Science Tools Ltd.
# Distributed under the terms of the GNU General Public License (GPL).
#


from pathlib import Path
from psychopy.localization import _translate
from psychopy.experiment import Param
from psychopy.experiment.routines._base import BaseStandaloneRoutine

_localized = {}

_localized.update({
    'path': _translate('Path')})


class BidsExportRoutine(BaseStandaloneRoutine):
    """An event class for inserting arbitrary code into Builder experiments"""

    categories = ['BIDS']
    targets = ['PsychoPy']
    iconFile = Path(__file__).parent / 'BIDS.png'
    tooltip = _translate('BIDS export: creates BIDS structure, writes tsv file and update further files')

    def __init__(self, exp, name='bids',
                 experiment_bids="bids",
                 data_type="beh",
                 acq="",
                 event_json=""
                 ):
        # Initialise base routine
        BaseStandaloneRoutine.__init__(self, exp, name=name)

        self.exp.requireImport(importName='BIDSHandler',
                               importFrom='psychopy_bids.bids')

        self.type = 'BIDSexport'

        # params
        # self.params = {}

        # Basic params
        self.order += [
            'data_type',
            'experiment_bids',
            'acq',
            'event_json'
        ]

        self.params['name'].hint = _translate("Name of the task. No two tasks should have the same name.")
        self.params['name'].label = _translate('task name')

        hnt = _translate("Name of the experiment (parent folder), if this (task) is part of a larger one.")
        self.params['experiment_bids'] = Param(
            experiment_bids, valType='str', inputType="single", categ="Basic", allowedTypes=[],
            canBePath=False,
            hint=hnt,
            label=_translate("experiment name"))

        hnt = _translate("BIDS defined data type")
        self.params['data_type'] = Param(
            data_type, valType='str', inputType="choice", categ='Basic',
            allowedVals=['func', 'dwi', 'fmap', 'anat', 'perf', 'meg', 'eeg', 'ieeg', 'beh', 'pet', 'micr'],
            hint=hnt,
            label=_translate("data type"))

        hnt = _translate("Custom label to distinguish different conditions present during"
                         " multiple runs of the same task")
        self.params['acq'] = Param(
            acq, valType='str', inputType="single", categ='Basic',
            allowedVals=[],
            canBePath=False,
            hint=hnt,
            label=_translate("acquisition mode"))

        hnt = _translate("Name of the default event JSON file. Will be copied into each subject folder.")
        self.params['event_json'] = Param(
            event_json, valType='str', inputType="single", categ='Basic',
            allowedVals=[],
            hint=hnt,
            label=_translate('event JSON'))

        # these inherited params are harmless but might as well trim:
        for parameter in ('startType', 'startVal', 'startEstim', 'stopVal',
                          'stopType', 'durationEstim',
                          'saveStartStop', 'syncScreenRefresh'):
            if parameter in self.params:
                del self.params[parameter]

    def writeStartCode(self, buff):
        """ Code for the start of the experiment """
        inits = self.params

        code = (
            "# create initial folder structure\n"
            "if expInfo['session']:\n"
        )
        buff.writeIndentedLines(code)
        buff.setIndentLevel(1, relative=True)
        code = (
            "bids_handler = BIDSHandler(dataset=%(experiment_bids)s,\n"
            " subject=expInfo['participant'], task=expInfo['expName'],\n"
            " session=expInfo['session'], data_type=%(data_type)s, acq=%(acq)s)\n"
        )
        buff.writeIndentedLines(code % inits)
        buff.setIndentLevel(-1, relative=True)
        code = (
            "else:"
        )
        buff.writeIndentedLines(code)
        buff.setIndentLevel(1, relative=True)
        code = (
            "bids_handler = BIDSHandler(dataset=%(experiment_bids)s,\n"
            " subject=expInfo['participant'], task=expInfo['expName'],\n"
            " data_type=%(data_type)s, acq=%(acq)s)\n"
        )
        buff.writeIndentedLines(code % inits)
        buff.setIndentLevel(-1, relative=True)
        code = (
            "bids_handler.createDataset()\n"
        )
        buff.writeIndentedLines(code % inits)

    def writeEndCode(self, buff):
        """write code at the end of the experiment"""
        inits = self.params

        code = (
            "# get participant_info and events from the ExperimentHandler\n"
            "ignore_list = ['participant', 'session', 'date', 'expName', 'psychopyVersion', 'OS', 'frameRate']\n"
            "participant_info = {key: thisExp.extraInfo[key] for key in thisExp.extraInfo if key not in ignore_list}\n"
            "# write tsv file and update\n"
            "event_file = bids_handler.addTaskEvents(thisExp.getAllEntries(), participant_info)\n"
            "bids_handler.addJSONSidecar(event_file, %(event_json)s, thisExp)\n"
            "bids_handler.addStimuliFolder(event_file)\n"
        )
        buff.writeIndentedLines(code % inits)



"""
    This module provides a class to handle the communication between the PsychoPy ExperimentHandler
    and the BIDS data structure
"""

import os
import glob
import json
import sys
import shutil
import platform
import re
import warnings

from ast import literal_eval
from datetime import datetime

import pandas as pd
import numpy as np

from . import BIDSTaskEvent


class BIDSHandler():
    """
        A class to handle the BIDSTaskEvents saved in the PsychoPy ExperimentHandler.

        Attributes
        ----------
        `dataset` : str
            A set of neuroimaging and behavioral data acquired for a purpose of a particular study.
        `subject` : str
            A person or animal participating in the study.
        `task` : int
            A set of structured activities performed by the participant.
        `session` : int
            A logical grouping of neuroimaging and behavioral data consistent across subjects.
        `data_type` : str
            A functional group of different types of data.
        `acq` : str
            Custom label to distinguish different conditions present during multiple runs of the
            same task
        """

    def __init__(self, dataset, subject, task, session=None, data_type='beh', acq=None):
        self.dataset = dataset
        self.subject = subject
        self.task = task
        self.session = session
        self.data_type = data_type
        self.acq = acq

# ----------------------------------------------------------------------------------------------- #

    def addJSONSidecar(self, event_file, existing_file=None, exp=None):
        """
        Add an accompanying JSON sidecar file for task event file documentation.

        Parameters
        ----------
        `event_file` : str
            path of the accompanying task event file.
        `existing_file` : str
            path to an existing sidecar JSON file.
        `exp` : ExperimentHandler
            a set of structured activities performed by the participant.

        Examples
        --------
        >>> handler.addJSONSidecar("sub-01_ses-1_task-simple_run-1_events.tsv", "temp.json", exp)
        """
        # Create a framework for the sidecar file
        if existing_file and existing_file != "":
            with open(existing_file, mode="r", encoding="utf-8") as json_reader:
                try:
                    sidecar = json.load(json_reader)
                except ValueError:
                    sys.exit("File MUST be a valid JSON file")
        else:
            sidecar = {}
            df = pd.read_csv(event_file, sep='\t')
            column_names = df.columns.values
            for name in column_names:
                sidecar[name] = {"LongName": "", "Description": ""}

        # Add Stimulus presentation details
        osSystem = f"{platform.system()} {platform.release()}"
        if exp and exp.extraInfo['psychopyVersion']:
            sidecar["StimulusPresentation"] = {"OperationgSystem": osSystem,
                                               "SoftwareName": "PsychoPy",
                                               "SoftwareRRID": "SCR_006571",
                                               "SoftwareVersion": exp.extraInfo['psychopyVersion']}
        else:
            sidecar["StimulusPresentation"] = {"OperationgSystem": osSystem}

        # Save the file in the same folder as the .tsv file
        file_pth = event_file.split('.')[0]
        if not os.path.exists(f'{file_pth}.json'):
            with open(f'{file_pth}.json', mode="w", encoding="utf-8") as json_file:
                json.dump(sidecar, json_file)

# ----------------------------------------------------------------------------------------------- #

    def addStimuliFolder(self, event_file):
        """
        Add a stimuli folder (under the root folder of the dataset)

        Parameters
        ----------
        `event_file` : str
            path of the accompanying task event file.

        Examples
        --------
        >>> handler.addStimuliFolder("sub-01_ses-1_task-simple_run-1_events.tsv")
        """
        df = pd.read_csv(event_file, sep='\t')
        if 'stim_file' in df.columns:
            stimuli = list(df['stim_file'].dropna().unique())
            if not os.path.exists(f'{self.dataset}{os.sep}stimuli'):
                os.makedirs(f'{self.dataset}{os.sep}stimuli')
            for st in stimuli:
                if os.path.isfile(st):
                    shutil.copyfile(st, f'{self.dataset}{os.sep}stimuli{os.sep}{st}')
                else:
                    warnings.warn(f"File {st} does not exist!")

# ----------------------------------------------------------------------------------------------- #

    def addTaskEvents(self, events, participant_info):
        """
        Add all task events of one task to the subject folder.

        Parameters
        ----------
        `task` : ExperimentHandler
            a set of structured activities performed by the participant.
        `participant_info` : dict
            a dictionary describing properties of each participant (such as age, sex, etc.)

        Return
        ------
        `file_name` : str
            path of the created tsv event file.

        Examples
        --------
        >>> event_list = [{'trigger': bids.BIDSTaskEvent(onset=1.0, duration=0}]
        >>> participant_info = {'participant_id': handler.subject, 'age': 18}
        >>> handler.addTaskEvents(event_list, participant_info)
        """
        participants_file = f'{self.dataset}{os.sep}participants.tsv'
        participant_info['participant_id'] = self.subject

        # Create the header of the tsv file and add the first subject
        if os.path.getsize(participants_file) == 0:
            df = pd.DataFrame()
            # Create dict for participants information
            participants = {}
            df = pd.concat([df, pd.DataFrame([participant_info])])
            for info in participant_info:
                participants.update({info: {"LongName": "", "Description": ""}})
            with open(f"{self.dataset}{os.sep}participants.json",
                      mode="w",
                      encoding="utf-8") as json_file:
                json.dump(participants, json_file)
            df.to_csv(participants_file, sep='\t', index=False)

        # Update the participants.tsv file
        else:
            df = pd.read_csv(participants_file, sep='\t')
            if self.subject not in df['participant_id'].tolist():
                df = df.append(participant_info, ignore_index=True)
                df = df.fillna('n/a')
                df.to_csv(participants_file, sep='\t', index=False)

        # Write the task events from the Experiment-Handler to the .tsv-file
        bids_events = []
        for entry in events:
            ev = [e.to_dict() for e in list(entry.values()) if isinstance(e, BIDSTaskEvent)]
            bids_events.extend(ev)

        # Set the path of the folder and the file
        if self.session:
            pth = (f'{self.dataset}{os.sep}'
                   f'{self.subject}{os.sep}'
                   f'{self.session}{os.sep}'
                   f'{self.data_type}')
        else:
            pth = f'{self.dataset}{os.sep}{self.subject}{os.sep}{self.data_type}'

        if not os.path.exists(pth):
            os.makedirs(pth)

        if self.session:
            file = f'{pth}{os.sep}{self.subject}_{self.session}_{self.task}'
        else:
            file = f'{pth}{os.sep}{self.subject}_{self.task}'

        if self.acq:
            file = "_".join([file, self.acq])

        run = len(glob.glob(f"{file}*_events.tsv")) + 1
        file_name = f'{file}_run-{run}_events.tsv'

        # Drop the empty columns and change None values to 'n/a'
        df = pd.DataFrame(bids_events)
        df.fillna(value=np.nan)
        df.dropna(how='all', axis=1, inplace=True)
        df = df.fillna('n/a')

        # Arrange the columns so that onset and duration are at the first two columns
        df = df[(['onset', 'duration'] + [col for col in df if col not in ['onset', 'duration']])]

        df.to_csv(file_name, sep='\t', index=False)
        return file_name

# ----------------------------------------------------------------------------------------------- #

    def createDataset(self):
        """
        Creates the rudimentary body of a new dataset.

        Examples
        --------
        >>> handler.createDataset()
        """
        if not os.path.exists(self.dataset):
            os.makedirs(self.dataset)
            with open(f'{self.dataset}{os.sep}participants.tsv', mode="w", encoding="utf-8"):
                pass

            bidsdir = sys.modules['psychopy_bids.bids'].__path__[0]
            readme_src = os.path.normpath(os.path.join(bidsdir, 'template', 'README'))
            readme_dest = os.path.join(self.dataset, 'README')
            shutil.copyfile(readme_src, readme_dest)
            ds_desc_src = os.path.normpath(os.path.join(bidsdir,
                                                        'template',
                                                        'dataset_description.json'))
            ds_desc_dest = os.path.join(self.dataset, 'dataset_description.json')
            shutil.copyfile(ds_desc_src, ds_desc_dest)

            with open(f'{self.dataset}{os.sep}CHANGES', mode="w", encoding="utf-8") as f:
                current_time = datetime.now()
                f.write(f"1.0.0 {current_time.year}-{current_time.month}-{current_time.day}\n")
                f.write(" - First initialization of the data set.\n")

# ----------------------------------------------------------------------------------------------- #

    def parseLog(self, file, level='BIDS', regex=None):
        """
        Updates a task with all task events (from a specific log file).

        Parameters
        ----------
            `file` : str
                file path of the log file.
            `level` : str
                level name of the bids task events.
            `regex` : str
                regular expression to parse the message string

        Return
        ------
            `events` : list
                a list of events like presented stimuli or participant responses

        Examples
        --------
        >>> events = handler.parseLog("simple.log", "BIDS")
        """

        events = []
        with open(file, mode="r", encoding="utf-8") as log_file:
            for line in log_file.readlines():
                event = re.split(r' \t|[ ]+', line, maxsplit=2)
                if event[1] == level:
                    if regex:
                        d = (dict(re.findall(regex, event[2])))
                    else:
                        d = {k: v for k, v in literal_eval(event[2]).items() if v is not None}
                    bids_event = BIDSTaskEvent(0, 0)
                    events.append({'bids_event': bids_event.from_dict(d)})
        return events

# ----------------------------------------------------------------------------------------------- #

    @property
    def dataset(self):
        """
        A set of neuroimaging and behavioral data acquired for a purpose of a particular study.
        """
        return self.__dataset

    @dataset.setter
    def dataset(self, dataset):
        self.__dataset = str(dataset)

    @property
    def subject(self):
        """
        A participant identifier of the form sub-<label>, matching a participant entity found in
        the dataset.
        """
        return self.__subject

    @subject.setter
    def subject(self, subject):
        match = re.match('^sub-[0-9a-zA-Z]+$', str(subject))
        if match:
            self.__subject = subject
        else:
            subject = re.sub('[^A-Za-z0-9]+', '', str(subject))
            self.__subject = f'sub-{subject}'

    @property
    def task(self):
        """
        A set of structured activities performed by the participant.
        """
        return self.__task

    @task.setter
    def task(self, task):
        regex = re.compile('^task-[0-9a-zA-Z]+$', re.I)
        match = regex.match(str(task))
        if match:
            self.__task = task
        else:
            task = re.sub('[^A-Za-z0-9]+', '', str(task))
            self.__task = f'task-{task}'

    @property
    def session(self):
        """
        A logical grouping of neuroimaging and behavioral data consistent across subjects.
        """
        return self.__session

    @session.setter
    def session(self, session):
        if session:
            regex = re.compile('^ses-[0-9a-zA-Z]+$', re.I)
            match = regex.match(str(session))
            if match:
                self.__session = session
            else:
                session = re.sub('[^A-Za-z0-9]+', '', str(session))
                self.__session = f'ses-{session}'
        else:
            self.__session = None

    @property
    def data_type(self):
        """
        A functional group of different types of data.
        """
        return self.__data_type

    @data_type.setter
    def data_type(self, data_type):
        dt = ['anat', 'beh', 'dwi', 'eeg', 'fmap', 'func', 'ieeg', 'meg', 'micr', 'perf', 'pet']
        msg = f"<data_type> MUST be one of the following: {dt}"
        if str(data_type) in dt:
            self.__data_type = str(data_type)
        else:
            sys.exit(msg)

    @property
    def acq(self):
        """
        A label to distinguish a different set of parameters used for acquiring the same modality.
        """
        return self.__acq

    @acq.setter
    def acq(self, acq):
        if acq:
            regex = re.compile('^acq-[0-9a-zA-Z]+$', re.I)
            match = regex.match(str(acq))
            if match:
                self.__acq = acq
            else:
                acq = re.sub('[^A-Za-z0-9]+', '', str(acq))
                self.__acq = f'acq-{acq}'
        else:
            self.__acq = None



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
    PsychoPy Builder component events
"""

# Part of the PsychoPy library
# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2021 Open Science Tools Ltd.
# Distributed under the terms of the GNU General Public License (GPL).

# from os import path
from pathlib import Path

from psychopy.experiment.components import BaseComponent, _translate, Param

# from psychopy import prefs

# only use _localized values for label values, nothing functional:
_localized = {'name': _translate('Name')}

_localized.update({'onset': _translate('Onset'),
                   'bids_duration': _translate('Duration'),
                   'trial_type': _translate('Trial type'),
                   'sample': _translate('Sample'),
                   'response_time': _translate('Response time'),
                   'value': _translate('Value'),
                   'hed': _translate('HED'),
                   'stim_file': _translate('Stimulus file'),
                   'identifier': _translate('Identifier'),
                   'database': _translate('Database'),
                   'add_log': _translate('Add to log file'),
                   })


class BidsEventComponent(BaseComponent):
    """An event class for inserting arbitrary code into Builder experiments"""

    categories = ['BIDS']
    targets = ['PsychoPy']  # , 'PsychoJS'
    iconFile = Path(__file__).parent / 'BIDS.png'
    tooltip = _translate('BIDS event: logging of BIDS events')

    def __init__(self, exp, parentName, name='bidsEvent',
                 onset=0, bids_duration=1, trial_type=None,
                 sample=None, response_time=None,
                 value=None, hed=None,
                 stim_file=None, identifier=None, database=None,
                 add_log=False):
        self.type = 'BIDSevent'
        self.exp = exp  # so we can access the experiment if necessary
        self.parentName = parentName  # to access the routine too if needed
        self.params = {}
        self.depends = []
        super(BidsEventComponent, self).__init__(
            exp, parentName, name=name)

        self.exp.requireImport(importName='BIDSTaskEvent',
                               importFrom='psychopy_bids.bids')

        # onset,duration,trial_type,sample,response_time,value,HED,stim_file,identifier,database

        _allow3 = ['constant']  # , 'set every repeat']  # list

        # Basic params
        self.order += ['onset', 'bids_duration', 'trial_type', 'response_time']

        hnt = _translate(
            "Onset (in seconds) of the event, measured from the beginning of the acquisition of the first data point"
            " stored in the corresponding task data file.")
        self.params['onset'] = Param(
            onset, valType='num', inputType="single", allowedTypes=[], categ='Basic',
            updates='constant', allowedUpdates=_allow3[:],  # copy the list
            hint=hnt,
            label=_localized['onset'])
        hnt = _translate(
            "Duration of the event (measured from onset) in seconds. Must always be either zero or positive (or n/a"
            " if unavailable).")
        self.params['bids_duration'] = Param(
            bids_duration, valType='num', inputType="single", allowedTypes=[], categ='Basic',
            updates='constant', allowedUpdates=_allow3[:],  # copy the list
            hint=hnt,
            label=_localized['bids_duration'])
        hnt = _translate(
            "Primary categorisation of each trial to identify them as instances of the experimental conditions.")
        self.params['trial_type'] = Param(
            trial_type, valType='str', inputType="single", allowedTypes=[], categ='Basic',
            updates='constant', allowedUpdates=_allow3[:],
            canBePath=False,
            hint=hnt,
            label=_localized['trial_type'])
        hnt = _translate(
            "Response time measured in seconds.")
        self.params['response_time'] = Param(
            response_time, valType='num', inputType="single", allowedTypes=[], categ='Basic',
            updates='constant', allowedUpdates=_allow3[:],  # copy the list
            hint=hnt,
            label=_localized['response_time'])

        # Data params
        hnt = _translate(
            "Should the event be saved in the log file too?")
        self.params['add_log'] = Param(
            add_log, valType='bool', inputType="bool", categ='Data',
            hint=hnt,
            label=_translate("Add event to log"))

        # Stim params
        self.order += ['stim_file', 'identifier', 'database']

        hnt = _translate(
            "Represents the location of the stimulus file. The values correspond to a path relative to /stimuli.")
        self.params['stim_file'] = Param(
            stim_file, valType='str', inputType="single", allowedTypes=[], categ='Stim',
            updates='constant', allowedUpdates=_allow3[:],
            hint=hnt,
            label=_localized['stim_file'])
        hnt = _translate(
            "References within a database")
        self.params['identifier'] = Param(
            identifier, valType='str', inputType="single", allowedTypes=[], categ='Stim',
            updates='constant', allowedUpdates=_allow3[:],
            canBePath=False,
            hint=hnt,
            label=_localized['identifier'])
        hnt = _translate(
            "References to a database")
        self.params['database'] = Param(
            database, valType='str', inputType="single", allowedTypes=[], categ='Stim',
            updates='constant', allowedUpdates=_allow3[:],
            canBePath=False,
            hint=hnt,
            label=_localized['database'])

        # more params
        self.order += ['sample', 'value', 'hed']

        hnt = _translate(
            "Onset of the event according to the sampling scheme of the recorded modality (that is, referring to the"
            " raw data file that the events.tsv file accompanies).")
        self.params['sample'] = Param(
            sample, valType='num', inputType="single", allowedTypes=[], categ='More',
            updates='constant', allowedUpdates=_allow3[:],  # copy the list
            hint=hnt,
            label=_localized['sample'])
        hnt = _translate(
            "Marker value associated with the event (for example, the value of a TTL trigger that was recorded at the"
            " onset of the event).")
        self.params['value'] = Param(
            value, valType='str', inputType="single", allowedTypes=[], categ='More',
            updates='constant', allowedUpdates=_allow3[:],
            canBePath=False,
            hint=hnt,
            label=_localized['value'])
        hnt = _translate(
            "Hierarchical Event Descriptor (HED) Tag")
        self.params['hed'] = Param(
            hed, valType='str', inputType="single", allowedTypes=[], categ='More',
            updates='constant', allowedUpdates=_allow3[:],
            canBePath=False,
            hint=hnt,
            label=_localized['hed'])

        # these inherited params are harmless but might as well trim:
        for parameter in ('startType', 'startVal', 'startEstim', 'stopVal',
                          'stopType', 'durationEstim',
                          'saveStartStop', 'syncScreenRefresh'):
            if parameter in self.params:
                del self.params[parameter]

    def writeStartCode(self, buff):
        """ write code in the start of the experimental code """
        # inits = self.params

        code = (
            "bidsLogLevel = 24\n"
            "logging.addLevel('BIDS', 24)\n"
        )
        if self.params['add_log']:
            buff.writeIndentedLines(code)

    def writeRoutineEndCode(self, buff):
        """
         write code
        """
        init_params = self.params
        params = ["trial_type", "sample", "response_time", "value", "hed", "stim_file", "identifier", "database"]
        # what loop are we in (or thisExp)?
        if len(self.exp.flow._loopList):
            curr_loop = self.exp.flow._loopList[-1]  # last (outer-most) loop
        else:
            curr_loop = self.exp._expHandler

        if 'Stair' in curr_loop.type:
            add_data_func = 'addOtherData'
        else:
            add_data_func = 'addData'

        loop = curr_loop.params['name']
        name = self.params['name']
        code = (
            "bids_event = BIDSTaskEvent( onset=%(onset)s,\n"
        )
        for parameter in params:
            if init_params[parameter] != 'None':
                code += (
                    " " + parameter + "=%(" + parameter + ")s,"
                )

        code += (
            " duration=%(bids_duration)s)\n"
        )
        buff.writeIndentedLines(code % init_params)
        code = (
            f"{loop}.{add_data_func}('{name}.event', bids_event)\n"
        )
        buff.writeIndentedLines(code)

        if self.params['add_log']:
            code = (
                "logging.log(level=24, msg=bids_event.to_dict())\n"
            )

            buff.writeIndentedLines(code)



"""
    Package to support the creation of valid bids-datasets.
"""

import os
import sys
import psychopy
import shutil

from importlib.metadata import version
__version__ = version("psychopy_bids")


def addBuilderElements():
    """
    With the help of this function the Bids Event component and Bids Export routine get copied to
    the correct psychopy path.

    This option is only temporarily supported until the psychopy plugin system is fully implemented.

    Examples
    --------
    >>> psychopy_bids.addBuilderElements()
    """
    src_path = os.path.dirname(__file__)
    try:
        dst_path = os.path.dirname(psychopy.__file__)
    except NameError:
        sys.exit("Psychopy is not installed!")
    try:
        shutil.copytree(f"{src_path}{os.sep}bids_event",
                        f"{dst_path}{os.sep}experiment{os.sep}components{os.sep}bids_event")
        shutil.copytree(f"{src_path}{os.sep}bids_settings",
                        f"{dst_path}{os.sep}experiment{os.sep}routines{os.sep}bids_settings")
    except FileExistsError:
        sys.exit("Bids Event and Bids Export are already added!")



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['psychopy_bids',
 'psychopy_bids.bids',
 'psychopy_bids.bids_event',
 'psychopy_bids.bids_settings']

package_data = \
{'': ['*'],
 'psychopy_bids': ['png/*'],
 'psychopy_bids.bids': ['template/*'],
 'psychopy_bids.bids_event': ['classic/*', 'dark/*', 'light/*'],
 'psychopy_bids.bids_settings': ['classic/*', 'dark/*', 'light/*']}

install_requires = \
['PsychoPy>=2022.2.3,<2023.0.0', 'pandas>=1.4.3,<2.0.0']

entry_points = \
{'psychopy': ['bids = psychopy_bids.bids'],
 'psychopy.experiment.components': ['bids_event = psychopy_bids.bids_event'],
 'psychopy.experiment.routines': ['bids_settings = '
                                  'psychopy_bids.bids_settings']}

setup_kwargs = {
    'name': 'psychopy-bids',
    'version': '0.1.2',
    'description': 'A PsychoPy plugin to work with the Brain Imaging Data Structure (BIDS)',
    'long_description': '# psychopy_bids\n\nA [PsychoPy](https://www.psychopy.org/) plugin to work with the [Brain Imaging Data Structure (BIDS)](https://bids-specification.readthedocs.io/)\n\n## Installation\n\n```bash\n$ pip install psychopy_bids\n```\n\n## Usage\n\n`psychopy_bids` can be used to create valid BIDS datasets by adding the possibility of using [task events](https://bids-specification.readthedocs.io/en/stable/04-modality-specific-files/05-task-events.html) in PsychoPy.\n\n```python\nfrom psychopy_bids import bids\n\nhandler = bids.BIDSHandler(dataset="example", subject="01", task="A")\nhandler.createDataset()\n\nevent_list = [{\'trigger\': bids.BIDSTaskEvent(onset=1.0, duration=0, trial_type="trigger")}]\nparticipant_info = {\'participant_id\': handler.subject, \'age\': 18}\n\nhandler.addTaskEvents(event_list, participant_info)\n```\n\n## Contributing\n\nInterested in contributing? Check out the contributing guidelines. Please note that this project is released with a Code of Conduct. By contributing to this project, you agree to abide by its terms.\n\n## License\n\n`psychopy_bids` was created by Christoph Anzengruber & Florian Schöngaßner. It is licensed under the terms of the GNU General Public License v3.0 license.\n\n## Credits\n\n`psychopy_bids` was created with [`cookiecutter`](https://cookiecutter.readthedocs.io/en/latest/) and the `py-pkgs-cookiecutter` [template](https://github.com/py-pkgs/py-pkgs-cookiecutter).\n',
    'author': 'Christoph Anzengruber & Florian Schöngaßner',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



"""This module provides a class to save your TaskEvents in the BIDS data structure"""

import sys


class BIDSTaskEvent(object):
    """
        A BIDSTaskEvent describes timing and other properties of events recorded during a run.
        Events are, for example, stimuli presented to the participant or participant responses.

        Attributes
        ----------
        onset : int, float
            Onset (in seconds) of the event, measured from the beginning of the acquisition of the
            first data point stored in the corresponding task data file.
        duration : int, float, 'n/a'
            Duration of the event (measured from onset) in seconds.
        trial_type : str
            Primary categorisation of each trial to identify them as instances of the experimental
            conditions.
        sample : int
            Onset of the event according to the sampling scheme of the recorded modality.
        response_time : int, float, 'n/a'
            Response time measured in seconds.
        value : Any
            Marker value associated with the event.
        hed : str
            Hierarchical Event Descriptor (HED) Tag.
        stim_file : str
            Represents the location of the stimulus file (such as an image, video, or audio file)
            presented at the given onset time.
        identifier : str
            Represents the database identifier of the stimulus file presented at the given onset
            time.
        database : str
            Represents the database of the stimulus file presented at the given onset time.
    """

    def __init__(self,
                 onset,
                 duration,
                 trial_type=None,
                 sample=None,
                 response_time=None,
                 value=None,
                 hed=None,
                 stim_file=None,
                 identifier=None,
                 database=None):
        self.onset = onset
        self.duration = duration
        self.trial_type = trial_type
        self.sample = sample
        self.response_time = response_time
        self.value = value
        self.hed = hed
        self.stim_file = stim_file
        self.identifier = identifier
        self.database = database

# ----------------------------------------------------------------------------------------------- #

    def __repr__(self):
        values = [v for v in self.__dict__.values() if v is not None]
        return f"BIDSTaskEvent({str(values)[1:-1]})"

# ----------------------------------------------------------------------------------------------- #

    def from_dict(self, dictionary):
        """
        Converts a dictionary into a BIDSTaskEvent object.

        Parameters
        ----------
        dictionary : dict
            a dictionary representing a task event.

        Returns
        -------
        BIDSTaskEvent
            BIDSTaskEvent object representing all column names of a task event

        Examples
        --------
        >>> event = BIDSTaskEvent(0, 0)
        >>> event.from_dict({'onset': 1, 'duration': 1})
        BIDSTaskEvent(1, 1)
        """
        for key in dictionary.keys():
            setattr(self, key, dictionary[key])
        return self

# ----------------------------------------------------------------------------------------------- #

    def to_dict(self):
        """
        Converts a BIDSTaskEvent object to a dictionary.

        Returns
        -------
        dict
            Dictionary representing all column names of a task event

        Examples
        --------
        >>> BIDSTaskEvent(1, 1).to_dict()
        {'onset': 1, 'duration': 1, 'trial_type': None, 'sample': None, 'response_time': None,
        'value': None, 'hed': None, 'stim_file': None, 'identifier': None, 'database': None}
        """
        return {"onset": self.onset,
                "duration": self.duration,
                "trial_type": self.trial_type,
                "sample": self.sample,
                "response_time": self.response_time,
                "value": self.value,
                "hed": self.hed,
                "stim_file": self.stim_file,
                "identifier": self.identifier,
                "database": self.database}

# ----------------------------------------------------------------------------------------------- #

    @property
    def onset(self):
        """
        Onset (in seconds) of the event, measured from the beginning of the acquisition of the
        first data point stored in the corresponding task data file.
        """
        return self.__onset

    @onset.setter
    def onset(self, onset):
        if isinstance(onset, (int, float)):
            self.__onset = round(onset, 4)
        elif isinstance(onset, str):
            try:
                self.__onset = round(float(onset), 4)
            except ValueError:
                sys.exit("Property 'onset' MUST be a number")
        else:
            sys.exit("Property 'onset' MUST be a number")

# ----------------------------------------------------------------------------------------------- #

    @property
    def duration(self):
        """
        Duration of the event (measured from onset) in seconds.
        """
        return self.__duration

    @duration.setter
    def duration(self, duration):
        msg = "Property 'duration' MUST be either zero or positive (or n/a if unavailable)"
        if isinstance(duration, str):
            if duration == 'n/a':
                self.__duration = duration
            else:
                try:
                    self.__duration = round(float(duration), 4)
                except ValueError:
                    sys.exit(msg)
        elif ((isinstance(duration, (int, float))) and (duration >= 0)):
            self.__duration = round(duration, 4)
        else:
            sys.exit(msg)

# ----------------------------------------------------------------------------------------------- #

    @property
    def trial_type(self):
        """
        Primary categorisation of each trial to identify them as instances of the experimental
        conditions.
        """
        return self.__trial_type

    @trial_type.setter
    def trial_type(self, trial_type):
        if trial_type:
            if isinstance(trial_type, str):
                self.__trial_type = trial_type
            else:
                sys.exit("Property 'trial_type' MUST be a string")
        else:
            self.__trial_type = None

# ----------------------------------------------------------------------------------------------- #

    @property
    def sample(self):
        """
        Onset of the event according to the sampling scheme of the recorded modality.
        """
        return self.__sample

    @sample.setter
    def sample(self, sample):
        if sample:
            if isinstance(sample, int):
                self.__sample = sample
            elif isinstance(sample, str):
                try:
                    self.__onset = int(sample)
                except ValueError:
                    sys.exit("Property 'sample' MUST be an integer")
                else:
                    sys.exit("Property 'sample' MUST be an integer")
        else:
            self.__sample = None

# ----------------------------------------------------------------------------------------------- #

    @property
    def response_time(self):
        """
        Response time measured in seconds.
        """
        return self.__response_time

    @response_time.setter
    def response_time(self, response_time):
        if response_time:
            msg = "Property 'response_time' MUST be a number (or n/a if unavailable)"
            if isinstance(response_time, str):
                if response_time == 'n/a':
                    self.__response_time = response_time
                else:
                    try:
                        self.__response_time = round(float(response_time), 4)
                    except ValueError:
                        sys.exit(msg)
            elif isinstance(response_time, (int, float)):
                self.__response_time = round(response_time, 4)
            else:
                sys.exit(msg)
        else:
            self.__response_time = None

# ----------------------------------------------------------------------------------------------- #

    @property
    def value(self):
        """
        Marker value associated with the event.
        """
        return self.__value

    @value.setter
    def value(self, value):
        if value:
            self.__value = value
        else:
            self.__value = None

# ----------------------------------------------------------------------------------------------- #

    @property
    def hed(self):
        """
        Hierarchical Event Descriptor (HED) Tag.
        """
        return self.__hed

    @hed.setter
    def hed(self, hed):
        if hed:
            if isinstance(hed, str):
                self.__hed = hed
            else:
                sys.exit("Property 'hed' MUST be a string")
        else:
            self.__hed = None

# ----------------------------------------------------------------------------------------------- #

    @property
    def stim_file(self):
        """
        Represents the location of the stimulus file (such as an image, video, or audio file)
        presented at the given onset time.
        """
        return self.__stim_file

    @stim_file.setter
    def stim_file(self, stim_file):
        if stim_file:
            if isinstance(stim_file, str):
                self.__stim_file = stim_file
            else:
                sys.exit("Property 'stim_file' MUST be a string")
        else:
            self.__stim_file = None

# ----------------------------------------------------------------------------------------------- #

    @property
    def identifier(self):
        """
        Represents the database identifier of the stimulus file presented at the given onset time.
        """
        return self.__identifier

    @identifier.setter
    def identifier(self, identifier):
        if identifier:
            if isinstance(identifier, str):
                self.__identifier = identifier
            else:
                sys.exit("Property 'identifier' MUST be a string")
        else:
            self.__identifier = None

# ----------------------------------------------------------------------------------------------- #

    @property
    def database(self):
        """
        Represents the database of the stimulus file presented at the given onset time.
        """
        return self.__database

    @database.setter
    def database(self, database):
        if database:
            if isinstance(database, str):
                self.__database = database
            else:
                sys.exit("Property 'database' MUST be a string")
        else:
            self.__database = None
